# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.address import Address
from ..types.address_response import AddressResponse
from ..types.create_address_request_geofence import CreateAddressRequestGeofence
from ..types.list_addresses_response import ListAddressesResponse
from ..types.update_address_request_geofence import UpdateAddressRequestGeofence
from .types.create_address_request_address_types_item import CreateAddressRequestAddressTypesItem
from .types.update_address_request_address_types_item import UpdateAddressRequestAddressTypesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAddressesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Address, ListAddressesResponse]:
        """
        Returns a list of all addresses in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Address, ListAddressesResponse]
            List of all addresses in the organization
        """
        _response = self._client_wrapper.httpx_client.request(
            "addresses",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "createdAfterTime": created_after_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListAddressesResponse,
                    parse_obj_as(
                        type_=ListAddressesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.pagination is not None:
                    _parsed_next = _parsed_response.pagination.end_cursor
                    _has_next = _parsed_next is not None and _parsed_next != ""
                    _get_next = lambda: self.list(
                        limit=limit,
                        after=_parsed_next,
                        parent_tag_ids=parent_tag_ids,
                        tag_ids=tag_ids,
                        created_after_time=created_after_time,
                        request_options=request_options,
                    )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        formatted_address: str,
        geofence: CreateAddressRequestGeofence,
        name: str,
        address_types: typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddressResponse]:
        """
        Creates a new address in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        formatted_address : str
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : CreateAddressRequestGeofence

        name : str
            Name of the address.

        address_types : typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddressResponse]
            Newly created address object with ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            "addresses",
            method="POST",
            json={
                "addressTypes": address_types,
                "contactIds": contact_ids,
                "externalIds": external_ids,
                "formattedAddress": formatted_address,
                "geofence": convert_and_respect_annotation_metadata(
                    object_=geofence, annotation=CreateAddressRequestGeofence, direction="write"
                ),
                "latitude": latitude,
                "longitude": longitude,
                "name": name,
                "notes": notes,
                "tagIds": tag_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[AddressResponse]:
        """
        Returns a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddressResponse]
            An Address.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        address_types: typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        formatted_address: typing.Optional[str] = OMIT,
        geofence: typing.Optional[UpdateAddressRequestGeofence] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddressResponse]:
        """
        Update a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        address_types : typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        formatted_address : typing.Optional[str]
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : typing.Optional[UpdateAddressRequestGeofence]

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        name : typing.Optional[str]
            Name of the address.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddressResponse]
            Updated address object with ID.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "addressTypes": address_types,
                "contactIds": contact_ids,
                "externalIds": external_ids,
                "formattedAddress": formatted_address,
                "geofence": convert_and_respect_annotation_metadata(
                    object_=geofence, annotation=UpdateAddressRequestGeofence, direction="write"
                ),
                "latitude": latitude,
                "longitude": longitude,
                "name": name,
                "notes": notes,
                "tagIds": tag_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAddressesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        created_after_time: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Address, ListAddressesResponse]:
        """
        Returns a list of all addresses in an organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        created_after_time : typing.Optional[str]
            A filter on data to have a created at time after or equal to this specified time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Address, ListAddressesResponse]
            List of all addresses in the organization
        """
        _response = await self._client_wrapper.httpx_client.request(
            "addresses",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "createdAfterTime": created_after_time,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListAddressesResponse,
                    parse_obj_as(
                        type_=ListAddressesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = False
                _get_next = None
                if _parsed_response.pagination is not None:
                    _parsed_next = _parsed_response.pagination.end_cursor
                    _has_next = _parsed_next is not None and _parsed_next != ""

                    async def _get_next():
                        return await self.list(
                            limit=limit,
                            after=_parsed_next,
                            parent_tag_ids=parent_tag_ids,
                            tag_ids=tag_ids,
                            created_after_time=created_after_time,
                            request_options=request_options,
                        )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        formatted_address: str,
        geofence: CreateAddressRequestGeofence,
        name: str,
        address_types: typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddressResponse]:
        """
        Creates a new address in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        formatted_address : str
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : CreateAddressRequestGeofence

        name : str
            Name of the address.

        address_types : typing.Optional[typing.Sequence[CreateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddressResponse]
            Newly created address object with ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "addresses",
            method="POST",
            json={
                "addressTypes": address_types,
                "contactIds": contact_ids,
                "externalIds": external_ids,
                "formattedAddress": formatted_address,
                "geofence": convert_and_respect_annotation_metadata(
                    object_=geofence, annotation=CreateAddressRequestGeofence, direction="write"
                ),
                "latitude": latitude,
                "longitude": longitude,
                "name": name,
                "notes": notes,
                "tagIds": tag_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AddressResponse]:
        """
        Returns a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddressResponse]
            An Address.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        address_types: typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]] = OMIT,
        contact_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        external_ids: typing.Optional[typing.Dict[str, str]] = OMIT,
        formatted_address: typing.Optional[str] = OMIT,
        geofence: typing.Optional[UpdateAddressRequestGeofence] = OMIT,
        latitude: typing.Optional[float] = OMIT,
        longitude: typing.Optional[float] = OMIT,
        name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        tag_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddressResponse]:
        """
        Update a specific address.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Addresses** under the Addresses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            ID of the Address. This can either be the Samsara-provided ID or an external ID. External IDs are customer-specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: `key:value`. For example, `crmId:abc123`

        address_types : typing.Optional[typing.Sequence[UpdateAddressRequestAddressTypesItem]]
            Reporting location type associated with the address (used for ELD reporting purposes). Valid values: `yard`, `shortHaul`, `workforceSite`, `riskZone`, `industrialSite`, `alertsOnly`, `agricultureSource`, `avoidanceZone`, `knownGPSJammingZone`, `authorizedZone`, `unauthorizedZone`, `vendor`, `inventory`.

        contact_ids : typing.Optional[typing.Sequence[str]]
            An array of Contact IDs associated with this Address.

        external_ids : typing.Optional[typing.Dict[str, str]]
            The [external IDs](https://developers.samsara.com/docs/external-ids) for the given object.

        formatted_address : typing.Optional[str]
            The full street address for this address/geofence, as it might be recognized by Google Maps.

        geofence : typing.Optional[UpdateAddressRequestGeofence]

        latitude : typing.Optional[float]
            Latitude of the address. Will be geocoded from `formattedAddress` if not provided.

        longitude : typing.Optional[float]
            Longitude of the address. Will be geocoded from `formattedAddress` if not provided.

        name : typing.Optional[str]
            Name of the address.

        notes : typing.Optional[str]
            Notes about the address.

        tag_ids : typing.Optional[typing.Sequence[str]]
            An array of IDs of tags to associate with this address.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddressResponse]
            Updated address object with ID.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"addresses/{jsonable_encoder(id)}",
            method="PATCH",
            json={
                "addressTypes": address_types,
                "contactIds": contact_ids,
                "externalIds": external_ids,
                "formattedAddress": formatted_address,
                "geofence": convert_and_respect_annotation_metadata(
                    object_=geofence, annotation=UpdateAddressRequestGeofence, direction="write"
                ),
                "latitude": latitude,
                "longitude": longitude,
                "name": name,
                "notes": notes,
                "tagIds": tag_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddressResponse,
                    parse_obj_as(
                        type_=AddressResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_approval_config_object_response_body_type import FormsApprovalConfigObjectResponseBodyType
from .forms_single_approval_config_object_response_body import FormsSingleApprovalConfigObjectResponseBody


class FormsApprovalConfigObjectResponseBody(UniversalBaseModel):
    """
    Form Template approval configuration object.
    """

    single_approval_config: typing_extensions.Annotated[
        typing.Optional[FormsSingleApprovalConfigObjectResponseBody],
        FieldMetadata(alias="singleApprovalConfig"),
        pydantic.Field(alias="singleApprovalConfig"),
    ] = None
    type: FormsApprovalConfigObjectResponseBodyType = pydantic.Field()
    """
    Type of approval.  Valid values: `singleApproval`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HosViolationDayObjectResponseBody(UniversalBaseModel):
    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description="The end time of the day on which the violation occurred in RFC 3339 format. This is determined by the driver's ELD start hour (00:00 or 12:00)",
        ),
    ]
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description="The start time of the day on which the violation occurred in RFC 3339 format. This is determined by the driver's ELD start hour (00:00 or 12:00)",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .safety_event_activity_feed_item_response_body_type import SafetyEventActivityFeedItemResponseBodyType
from .safety_event_object_response_body import SafetyEventObjectResponseBody


class SafetyEventActivityFeedItemResponseBody(UniversalBaseModel):
    """
    Safety event activity feed item.
    """

    id: str = pydantic.Field()
    """
    The ID of the activity event feed line item.
    """

    safety_event: typing_extensions.Annotated[
        SafetyEventObjectResponseBody, FieldMetadata(alias="safetyEvent"), pydantic.Field(alias="safetyEvent")
    ]
    time: str = pydantic.Field()
    """
    The time the activity occurred in the corresponding safety event in RFC 3339 milliseconds format.
    """

    type: SafetyEventActivityFeedItemResponseBodyType = pydantic.Field()
    """
    The type of activity that occurred in the safety event. We currently only support CoachingStateActivityType, BehaviorLabelActivityType, and CreateSafetyEventActivityType, but we may add support for more activity types in the future.  Valid values: `BehaviorLabelActivityType`, `CoachingStateActivityType`, `CreateSafetyEventActivityType`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .eld_settings_object_response_body import EldSettingsObjectResponseBody


class DriverWithTimezoneEldSettingsObjectResponseBody(UniversalBaseModel):
    """
    The driver the log applies to.
    """

    eld_settings: typing_extensions.Annotated[
        typing.Optional[EldSettingsObjectResponseBody],
        FieldMetadata(alias="eldSettings"),
        pydantic.Field(alias="eldSettings"),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the driver
    """

    name: str = pydantic.Field()
    """
    Name of the driver
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Home terminal timezone, in order to indicate what time zone should be used to calculate the ELD logs. Driver timezones use [IANA timezone database](https://www.iana.org/time-zones) keys (e.g. `America/Los_Angeles`, `America/New_York`, `Europe/London`, etc.). You can find a mapping of common timezone formats to IANA timezone keys [here](https://unicode.org/cldr/charts/latest/supplemental/zone_tzid.html).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

RouteEventResponseResponseBodyEventType = typing.Union[
    typing.Literal[
        "stopArrived",
        "stopCompleted",
        "stopEnRoute",
        "stopSkipped",
        "stopTaskCompleted",
        "stopTaskSkipped",
        "stopEtaUpdated",
        "unspecified",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ifta_detail_job_args_response_body import IftaDetailJobArgsResponseBody
from .ifta_detail_job_output_response_body import IftaDetailJobOutputResponseBody
from .ifta_detail_job_response_body_job_status import IftaDetailJobResponseBodyJobStatus


class IftaDetailJobResponseBody(UniversalBaseModel):
    """
    A job representing the async generation of IFTA mileage segments. The job should be polled until the state is terminal.
    """

    args: IftaDetailJobArgsResponseBody
    completed_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="completedAtTime"),
        pydantic.Field(alias="completedAtTime", description=" The time this job completed successfully."),
    ] = None
    details: typing.Optional[str] = pydantic.Field(default=None)
    """
    A message containing additional details about the job, if available.
    """

    failed_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="failedAtTime"),
        pydantic.Field(alias="failedAtTime", description=" The time this job failed."),
    ] = None
    files: typing.Optional[typing.List[IftaDetailJobOutputResponseBody]] = pydantic.Field(default=None)
    """
    A list of files generated by this job. This field will be omitted when the job status is "Failed" or no data exists with the provided job arguments. Read about the file structure [here](https://developers.samsara.com/docs/ifta-detail-csv).
    """

    job_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="jobId"),
        pydantic.Field(alias="jobId", description=" The ID used to retrieve this job."),
    ]
    job_status: typing_extensions.Annotated[
        IftaDetailJobResponseBodyJobStatus,
        FieldMetadata(alias="jobStatus"),
        pydantic.Field(
            alias="jobStatus",
            description=" The status of this job. `jobStatus` can be one of Requested, Processing, Completed, or Failed.  Valid values: `Requested`, `Processing`, `Completed`, `Failed`",
        ),
    ]
    requested_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="requestedAtTime"),
        pydantic.Field(alias="requestedAtTime", description=" The time this job was requested for creation."),
    ]
    started_at_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="startedAtTime"),
        pydantic.Field(alias="startedAtTime", description=" The time this job started processing."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .assets_location_link_config_address_details_object_response_body import (
    AssetsLocationLinkConfigAddressDetailsObjectResponseBody,
)
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class AssetsLocationLinkResponseConfigObjectResponseBody(UniversalBaseModel):
    """
    Configuration details specific to the 'By Asset' Live Sharing Link.
    """

    asset_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assetId"),
        pydantic.Field(alias="assetId", description="Unique assets ID that Live Sharing Link will show."),
    ] = None
    location: typing.Optional[AssetsLocationLinkConfigAddressDetailsObjectResponseBody] = None
    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with assets for Live Sharing Link. Only populated if the link is configured for by tags instead of by asset ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.training_courses_get_training_courses_response_body import TrainingCoursesGetTrainingCoursesResponseBody
from .raw_client import AsyncRawTrainingCoursesClient, RawTrainingCoursesClient


class TrainingCoursesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTrainingCoursesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTrainingCoursesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTrainingCoursesClient
        """
        return self._raw_client

    def get_training_courses(
        self,
        *,
        after: typing.Optional[str] = None,
        course_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        category_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        status: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrainingCoursesGetTrainingCoursesResponseBody:
        """
        Returns all training courses data. Results are paginated.
         Courses in the ‘draft’ status are excluded from the data returned by this endpoint.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Training Courses** under the Training Courses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        course_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated course IDs. If course ID is present, training assignments for the specified course ID(s) will be returned. Max value for this value is 100 objects. Defaults to returning all courses. Example: `courseIds=a4db8702-79d5-4396-a717-e301d52ecc11,c6490f6a-d84e-49b5-b0ad-b6baae304075`

        category_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated course category IDs. If courseCategoryId is present, training courses for the specified course category(s) will be returned. Max value for this value is 100 objects. Defaults to returning all courses.  Example: `categoryIds=a4db8702-79d5-4396-a717-e301d52ecc11,c6490f6a-d84e-49b5-b0ad-b6baae304075`

        status : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values. If status is present, training courses with the specified status(s) will be returned. Valid values: “published”, “deleted”, “archived”. Defaults to returning all courses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrainingCoursesGetTrainingCoursesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.training_courses.get_training_courses()
        """
        _response = self._raw_client.get_training_courses(
            after=after,
            course_ids=course_ids,
            category_ids=category_ids,
            status=status,
            request_options=request_options,
        )
        return _response.data


class AsyncTrainingCoursesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTrainingCoursesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTrainingCoursesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTrainingCoursesClient
        """
        return self._raw_client

    async def get_training_courses(
        self,
        *,
        after: typing.Optional[str] = None,
        course_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        category_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        status: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TrainingCoursesGetTrainingCoursesResponseBody:
        """
        Returns all training courses data. Results are paginated.
         Courses in the ‘draft’ status are excluded from the data returned by this endpoint.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Training Courses** under the Training Courses category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        course_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated course IDs. If course ID is present, training assignments for the specified course ID(s) will be returned. Max value for this value is 100 objects. Defaults to returning all courses. Example: `courseIds=a4db8702-79d5-4396-a717-e301d52ecc11,c6490f6a-d84e-49b5-b0ad-b6baae304075`

        category_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated course category IDs. If courseCategoryId is present, training courses for the specified course category(s) will be returned. Max value for this value is 100 objects. Defaults to returning all courses.  Example: `categoryIds=a4db8702-79d5-4396-a717-e301d52ecc11,c6490f6a-d84e-49b5-b0ad-b6baae304075`

        status : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Optional string of comma separated values. If status is present, training courses with the specified status(s) will be returned. Valid values: “published”, “deleted”, “archived”. Defaults to returning all courses.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TrainingCoursesGetTrainingCoursesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.training_courses.get_training_courses()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_training_courses(
            after=after,
            course_ids=course_ids,
            category_ids=category_ids,
            status=status,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

UsDriverRulesetOverrideRestbreak = typing.Union[
    typing.Literal["Property (off-duty/sleeper)", "California Mealbreak (off-duty/sleeper)", "None"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .location_response_response_body import LocationResponseResponseBody
from .safety_event_dismissal_reason_response_body import SafetyEventDismissalReasonResponseBody
from .safety_event_speeding_metadata_response_body import SafetyEventSpeedingMetadataResponseBody
from .safety_event_v_2_asset_object_response_body import SafetyEventV2AssetObjectResponseBody
from .safety_event_v_2_behavior_labels_response_body import SafetyEventV2BehaviorLabelsResponseBody
from .safety_event_v_2_context_labels_response_body import SafetyEventV2ContextLabelsResponseBody
from .safety_event_v_2_driver_object_response_body import SafetyEventV2DriverObjectResponseBody
from .safety_event_v_2_media_response_body import SafetyEventV2MediaResponseBody
from .safety_event_v_2_object_response_body_event_state import SafetyEventV2ObjectResponseBodyEventState


class SafetyEventV2ObjectResponseBody(UniversalBaseModel):
    """
    Full Safety Event object
    """

    asset: SafetyEventV2AssetObjectResponseBody
    assigned_coach: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="assignedCoach"),
        pydantic.Field(alias="assignedCoach", description="Unique user ID for the assigned coach."),
    ] = None
    behavior_labels: typing_extensions.Annotated[
        typing.List[SafetyEventV2BehaviorLabelsResponseBody],
        FieldMetadata(alias="behaviorLabels"),
        pydantic.Field(
            alias="behaviorLabels",
            description="The most up-to-date behavior labels associated with the safety event. These labels can be updated by Safety Admins.",
        ),
    ]
    context_labels: typing_extensions.Annotated[
        typing.List[SafetyEventV2ContextLabelsResponseBody],
        FieldMetadata(alias="contextLabels"),
        pydantic.Field(
            alias="contextLabels",
            description="The most up-to-date context labels associated with the safety event. AI generated labels can be updated by Safety Admins.",
        ),
    ]
    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="UTC time the Safety Event was created in Samsara in RFC 3339 format."
        ),
    ]
    detected_streams: typing_extensions.Annotated[
        typing.Optional[typing.List[SafetyEventV2MediaResponseBody]],
        FieldMetadata(alias="detectedStreams"),
        pydantic.Field(alias="detectedStreams", description="Camera streams that detected the safety event."),
    ] = None
    dismissal_reason: typing_extensions.Annotated[
        typing.Optional[SafetyEventDismissalReasonResponseBody],
        FieldMetadata(alias="dismissalReason"),
        pydantic.Field(alias="dismissalReason"),
    ] = None
    driver: SafetyEventV2DriverObjectResponseBody
    end_ms: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endMs"),
        pydantic.Field(alias="endMs", description="UTC time the Safety Event ended in RFC 3339 format."),
    ]
    event_state: typing_extensions.Annotated[
        SafetyEventV2ObjectResponseBodyEventState,
        FieldMetadata(alias="eventState"),
        pydantic.Field(
            alias="eventState",
            description="The current state of the Safety Event.  Valid values: `unknown`, `needsReview`, `reviewed`, `needsCoaching`, `coached`, `dismissed`, `needsRecognition`, `recognized`",
        ),
    ]
    id: str = pydantic.Field()
    """
    The unique Samsara ID (uuid) of the safety event.
    """

    inbox_event_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="inboxEventUrl"),
        pydantic.Field(
            alias="inboxEventUrl", description="A link to the Safety Event review page for the associated event."
        ),
    ]
    incident_report_url: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="incidentReportUrl"),
        pydantic.Field(
            alias="incidentReportUrl",
            description="If a harsh event, returns the URL of the associated incident report page. If a speeding event, returns the URL of the associated speeding report page.",
        ),
    ]
    location: LocationResponseResponseBody
    max_acceleration_g_force: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="maxAccelerationGForce"),
        pydantic.Field(
            alias="maxAccelerationGForce",
            description="The maximum acceleration value as a multiplier on the force of gravity (g).",
        ),
    ]
    media: typing.Optional[typing.List[SafetyEventV2MediaResponseBody]] = pydantic.Field(default=None)
    """
    Media assets available for the safety event.
    """

    speeding_metadata: typing_extensions.Annotated[
        typing.Optional[SafetyEventSpeedingMetadataResponseBody],
        FieldMetadata(alias="speedingMetadata"),
        pydantic.Field(alias="speedingMetadata"),
    ] = None
    start_ms: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startMs"),
        pydantic.Field(alias="startMs", description="UTC time the Safety Event started in RFC 3339 format."),
    ]
    trip_end_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tripEndTime"),
        pydantic.Field(
            alias="tripEndTime",
            description="UTC time the trip ended in RFC 3339 format. Null when Safety Event occurs off-trip or the trip is ongoing.",
        ),
    ] = None
    trip_start_time: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tripStartTime"),
        pydantic.Field(
            alias="tripStartTime",
            description="UTC time the trip started in RFC 3339 format. Null when Safety Event occurs off-trip.",
        ),
    ] = None
    updated_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="UTC time the Safety Event was updated in Samsara in RFC 3339 format."
        ),
    ]
    updated_by_user_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="updatedByUserId"),
        pydantic.Field(
            alias="updatedByUserId",
            description="The user ID associated with the user who made the event state change. Only returned if the event state changes.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_stats_fault_codes_value_j_1939 import VehicleStatsFaultCodesValueJ1939
from .vehicle_stats_fault_codes_value_obdii import VehicleStatsFaultCodesValueObdii
from .vehicle_stats_fault_codes_value_oem import VehicleStatsFaultCodesValueOem


class VehicleStatsFaultCodesValue(UniversalBaseModel):
    """
    Fault codes for the vehicle
    """

    can_bus_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="canBusType"),
        pydantic.Field(alias="canBusType", description="The CAN bus type of the vehicle."),
    ] = None
    j_1939: typing_extensions.Annotated[
        typing.Optional[VehicleStatsFaultCodesValueJ1939], FieldMetadata(alias="j1939"), pydantic.Field(alias="j1939")
    ] = None
    obdii: typing.Optional[VehicleStatsFaultCodesValueObdii] = None
    oem: typing.Optional[VehicleStatsFaultCodesValueOem] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_attribute_tiny_response_body import GoaAttributeTinyResponseBody
from .goa_gateway_tiny_response_response_body import GoaGatewayTinyResponseResponseBody
from .goa_tag_tiny_response_response_body import GoaTagTinyResponseResponseBody


class EquipmentWithAttributesResponseObjectResponseBody(UniversalBaseModel):
    """
    The equipment object.
    """

    attributes: typing.Optional[typing.List[GoaAttributeTinyResponseBody]] = pydantic.Field(default=None)
    """
    List of attributes associated with the entity
    """

    equipment_serial_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="equipmentSerialNumber"),
        pydantic.Field(alias="equipmentSerialNumber", description="The serial number of the equipment."),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique Samsara ID of the Equipment. This is automatically generated when the Equipment object is created. It cannot be changed.
    """

    installed_gateway: typing_extensions.Annotated[
        typing.Optional[GoaGatewayTinyResponseResponseBody],
        FieldMetadata(alias="installedGateway"),
        pydantic.Field(alias="installedGateway"),
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The human-readable name of the Equipment. This is set by a fleet administrator and will appear in both Samsara’s cloud dashboard as well as the Samsara Driver mobile app. By default, this name is the serial number of the Samsara Asset Gateway. It can be set or updated through the Samsara Dashboard or through the API at any time.
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    These are generic notes about the Equipment. Empty by default. Can be set or updated through the Samsara Dashboard or the API at any time.
    """

    tags: typing.Optional[typing.List[GoaTagTinyResponseResponseBody]] = pydantic.Field(default=None)
    """
    The list of [tags](https://kb.samsara.com/hc/en-us/articles/360026674631-Using-Tags-and-Tag-Nesting) associated with the Equipment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HarshBrakeSensitivityGForceSettingsObjectResponseBody(UniversalBaseModel):
    """
    The harsh brake sensitivity settings.
    """

    heavy_duty: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="heavyDuty"),
        pydantic.Field(alias="heavyDuty", description="Harsh brake sensitivity settings for heavy duty vehicle."),
    ] = None
    light_duty: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="lightDuty"),
        pydantic.Field(alias="lightDuty", description="Harsh brake sensitivity settings for light duty vehicle."),
    ] = None
    passenger: typing.Optional[str] = pydantic.Field(default=None)
    """
    Harsh brake sensitivity settings for passenger car.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionStepResultsItemFindCopies(UniversalBaseModel):
    angle_tolerance: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="angleTolerance"),
        pydantic.Field(alias="angleTolerance", description="The orientation angle tolerance (+/- °)"),
    ] = None
    found_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="foundCount"),
        pydantic.Field(alias="foundCount", description="The number of copies found"),
    ] = None
    max_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="maxCount"),
        pydantic.Field(alias="maxCount", description="The maximum number of copies allowed"),
    ] = None
    min_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="minCount"),
        pydantic.Field(alias="minCount", description="The minimum number of copies allowed"),
    ] = None
    threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    The minimum required similarity (in %) of a found copy compared to the configured match region
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .vehicle_aux_input_name import VehicleAuxInputName
from .vehicle_stats_aux_input_time import VehicleStatsAuxInputTime
from .vehicle_stats_decorations import VehicleStatsDecorations


class VehicleStatsAuxInputWithDecoration(UniversalBaseModel):
    """
    Data for auxiliary digio equipment.
    """

    decorations: typing.Optional[VehicleStatsDecorations] = None
    name: typing.Optional[VehicleAuxInputName] = None
    time: typing.Optional[VehicleStatsAuxInputTime] = None
    value: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean indicating the state of the auxiliary equipment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .equipment_time import EquipmentTime


class EquipmentEngineSeconds(UniversalBaseModel):
    """
    [DEPRECATED] Please use either `gatewayEngineSeconds` or `obdEngineSeconds`.
    """

    time: EquipmentTime
    value: int = pydantic.Field()
    """
    An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power and an offset provided manually through the Samsara cloud dashboard.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DriverEfficiencyPercentageDataObjectResponseBody(UniversalBaseModel):
    """
    Driver Efficiency percentage data. This object is returned when the “percentage” format is specified in “dataFormats”.
    """

    anticipation_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="anticipationPercentage"),
        pydantic.Field(
            alias="anticipationPercentage",
            description="Percentage of time a driver is in quickly breaking events vs total breaking events.",
        ),
    ] = None
    coasting_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="coastingPercentage"),
        pydantic.Field(alias="coastingPercentage", description="Percentage of time a driver is in coasting."),
    ] = None
    cruise_control_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="cruiseControlPercentage"),
        pydantic.Field(
            alias="cruiseControlPercentage", description="Percentage of time a vehicle is in cruise control."
        ),
    ] = None
    green_band_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="greenBandPercentage"),
        pydantic.Field(
            alias="greenBandPercentage", description="Percentage of time a driver is driving within the green band."
        ),
    ] = None
    high_grade_road_driving_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="highGradeRoadDrivingPercentage"),
        pydantic.Field(
            alias="highGradeRoadDrivingPercentage",
            description="Percentage of time a driver is driving on high-grade road.",
        ),
    ] = None
    high_torque_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="highTorquePercentage"),
        pydantic.Field(
            alias="highTorquePercentage", description="Percentage of time a driver is driving in high torque."
        ),
    ] = None
    idling_percentage: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="idlingPercentage"),
        pydantic.Field(alias="idlingPercentage", description="Percentage of time a driver is idling."),
    ]
    over_speed_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="overSpeedPercentage"),
        pydantic.Field(alias="overSpeedPercentage", description="Percentage of time a driver is in over-speeding."),
    ] = None
    wear_free_brake_percentage: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="wearFreeBrakePercentage"),
        pydantic.Field(
            alias="wearFreeBrakePercentage", description="Percentage of time a driver is in wear-free breaking."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .work_order_money_object_request_body import WorkOrderMoneyObjectRequestBody


class WorkOrderDiscountObjectRequestBody(UniversalBaseModel):
    """
    How much a work order is discounted. Either money or basisPoints are specified.
    """

    basis_points: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="basisPoints"),
        pydantic.Field(alias="basisPoints", description="The discount in basis points. 100 basis points = 1%."),
    ] = None
    money: typing.Optional[WorkOrderMoneyObjectRequestBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
